<?php

namespace App\Http\Controllers;

use App\Models\InsuranceClaim;
use App\Models\AdditionalInformation;
use App\Models\StatusChange;
use App\Models\CaseNote;
use App\Models\InsuranceClaimEdit;
use App\Models\CaseClosure;
use App\Models\ClaimAttachment;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use ZipArchive;
use File;
use Barryvdh\DomPDF\Facade\Pdf;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        // If user is an agent, only show open cases
        if (auth()->user()->role === 'agent') {
            $totalCases = InsuranceClaim::where('status', '!=', 'closed')->count();
            $loggedCases = InsuranceClaim::where('status', 'logged')->count();
            $inProcessCases = InsuranceClaim::where('status', 'in_process')->count();
            $openCases = InsuranceClaim::where('status', 'open')->count();
            $closedCases = 0; // Agents don't see closed cases
        } else {
            $totalCases = InsuranceClaim::count();
            $loggedCases = InsuranceClaim::where('status', 'logged')->count();
            $inProcessCases = InsuranceClaim::where('status', 'in_process')->count();
            $openCases = InsuranceClaim::where('status', 'open')->count();
            $closedCases = InsuranceClaim::where('status', 'closed')->count();
        }

        // Calculate percentages
        $loggedPercent = $totalCases > 0 ? round(($loggedCases / $totalCases) * 100) : 0;
        $inProcessPercent = $totalCases > 0 ? round(($inProcessCases / $totalCases) * 100) : 0;
        $openPercent = $totalCases > 0 ? round(($openCases / $totalCases) * 100) : 0;
        $closedPercent = $totalCases > 0 ? round(($closedCases / $totalCases) * 100) : 0;

        return view('pages.index', compact(
            'totalCases',
            'loggedCases',
            'inProcessCases',
            'openCases',
            'closedCases',
            'loggedPercent',
            'inProcessPercent',
            'openPercent',
            'closedPercent'
        ));
    }

    public function caseList()
    {
        // If user is an agent, only show non-closed cases
        if (auth()->user()->role === 'agent') {
            $allCases = InsuranceClaim::with('attachments')
                ->where('status', '!=', 'closed')
                ->latest()
                ->get();
        } else {
            $allCases = InsuranceClaim::with('attachments')->latest()->get();
        }

        // Filter cases based on status
        $loggedCases = $allCases->where('status', InsuranceClaim::STATUS_LOGGED);
        $openCases = $allCases->where('status', InsuranceClaim::STATUS_OPEN);
        $closedCases = $allCases->where('status', InsuranceClaim::STATUS_CLOSED);
        $reopenedCases = $allCases->where('reopen', 1);

        return view('pages.list', compact(
            'allCases',
            'loggedCases',
            'openCases',
            'closedCases',
            'reopenedCases'
        ));
    }

    public function caseDetail($id)
    {
        $case = InsuranceClaim::with(['attachments'])->findOrFail($id);
        $notes = CaseNote::where('insurance_claim_id', $id)
                        ->with('user')
                        ->orderBy('created_at', 'desc')
                        ->get();

        // Get edit history with user information
        $editHistory = InsuranceClaimEdit::where('insurance_claim_id', $id)
                        ->with('user')
                        ->orderBy('created_at', 'desc')
                        ->get();

        // Get additional information with dashcam footage
        $additionalInfo = AdditionalInformation::where('insurance_claim_id', $id)->first();

        // Get case closure information
        $caseClosure = CaseClosure::where('insurance_claim_id', $id)->first();

        return view('pages.details', compact('case', 'notes', 'editHistory', 'additionalInfo', 'caseClosure'));
    }

    public function storeAdditionalInfo(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'insurance_claim_id' => 'required|exists:insurance_claims,id',
            'policy_number' => 'required|string',
            'claim_number' => 'required|string',
            'incident_date' => 'required|date',
            'notified_date' => 'required|date',
            'driver_name' => 'required|string',
            'driver_age' => 'required|integer',
            'claim_notified_by' => 'required|string',
            'vehicle_registration' => 'required|string',
            'file_handler' => 'required|string',
            'responsibility' => 'required|string',
            'dashcam_footage' => 'nullable|file',
            'circumstances' => 'required|string',
            'digital_signature' => 'required|string'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'error',
                'errors' => $validator->errors()
            ], 422);
        }

        try {
            $data = $request->all();
            $data['user_id'] = auth()->id();

            if ($request->hasFile('dashcam_footage')) {
                $file = $request->file('dashcam_footage');
                $filename = time() . '_' . $file->getClientOriginalName();
                $file->move(public_path('uploads/dashcam'), $filename);
                $data['dashcam_footage'] = 'uploads/dashcam/' . $filename;
            }

            AdditionalInformation::create($data);

            return response()->json([
                'status' => 'success',
                'message' => 'Additional information added successfully'
            ], 200);
        } catch (\Exception $e) {
            Log::error('Error saving additional information: ' . $e->getMessage());
            return response()->json([
                'status' => 'error',
                'message' => 'An error occurred while saving the information: ' . $e->getMessage()
            ], 500);
        }
    }

    public function updateStatus(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'claim_id' => 'required|exists:insurance_claims,id',
            'from_status' => 'required|string',
            'to_status' => 'required|string|in:open,closed'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'error',
                'errors' => $validator->errors()
            ], 422);
        }

        try {
            $claim = InsuranceClaim::findOrFail($request->claim_id);

            // Verify current status matches from_status
            if ($claim->status !== $request->from_status) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Current status does not match. Please refresh the page.'
                ], 422);
            }

            // Update claim status
            $claim->status = $request->to_status;
            $claim->save();

            // Record status change
            StatusChange::create([
                'insurance_claim_id' => $request->claim_id,
                'user_id' => auth()->id(),
                'from_status' => $request->from_status,
                'to_status' => $request->to_status
            ]);

            return response()->json([
                'status' => 'success',
                'message' => 'Case status updated successfully'
            ]);
        } catch (\Exception $e) {
            \Log::error('Error updating case status: ' . $e->getMessage());
            return response()->json([
                'status' => 'error',
                'message' => 'An error occurred while updating the status'
            ], 500);
        }
    }

    public function storeNotes(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'insurance_claim_id' => 'required|exists:insurance_claims,id',
            'title' => 'required|string',
            'note_date' => 'required|date',
            'document' => 'nullable|file',
            'notes' => 'required|string',
            'digital_signature' => 'required|string'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => 'error',
                'errors' => $validator->errors()
            ], 422);
        }

        try {
            $data = $request->all();
            $data['user_id'] = auth()->id();

            if ($request->hasFile('document')) {
                $file = $request->file('document');
                $filename = time() . '_' . $file->getClientOriginalName();
                $file->move(public_path('uploads/notes'), $filename);
                $data['document'] = 'uploads/notes/' . $filename;
            }

            CaseNote::create($data);

            return response()->json([
                'status' => 'success',
                'message' => 'Notes added successfully'
            ]);
        } catch (\Exception $e) {
            \Log::error('Error saving notes: ' . $e->getMessage());
            return response()->json([
                'status' => 'error',
                'message' => 'An error occurred while saving the notes'
            ], 500);
        }
    }

    public function editCase(Request $request)
    {
        try {
            $case = InsuranceClaim::findOrFail($request->id);
            $case->update($request->all());

            // Log the edit
            InsuranceClaimEdit::create([
                'insurance_claim_id' => $case->id,
                'user_id' => auth()->id()
            ]);

            return redirect()->back()->with('success', 'Case updated successfully');

        } catch (\Exception $e) {
            \Log::error('Error updating case: ' . $e->getMessage());
            return redirect()->back()->with('error', 'An error occurred while updating the case');
        }
    }

    public function storeClosure(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'claim_id' => 'required|exists:insurance_claims,id',
                'outcome' => 'required|in:Faulty,Non Faulty',
                'final_costing' => 'required|numeric',
                'payment_date' => 'required|date',
                'case_close_date' => 'required|date',
                'closure_reasoning' => 'required|in:Settled,Denied,No Further Action,Withdrawn,Resolved by Agreement,Settled Privately',
                'supporting_documentation' => 'required|file',
                'brief_summary' => 'required|string',
                'outstanding_issues' => 'required|string',
                'communication_summary' => 'required|string',
                'final_case_notes' => 'required|string',
                'digital_signature' => 'required|string'
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Validation failed',
                    'errors' => $validator->errors()
                ], 422);
            }

            // Store the supporting documentation
            $file = $request->file('supporting_documentation');
            $path = $file->store('case-closures', 'public');

            // Create case closure record
            $closure = CaseClosure::create([
                'insurance_claim_id' => $request->claim_id,
                'user_id' => auth()->id(),
                'outcome' => $request->outcome,
                'final_costing' => $request->final_costing,
                'payment_date' => $request->payment_date,
                'case_close_date' => $request->case_close_date,
                'closure_reasoning' => $request->closure_reasoning,
                'supporting_documentation' => $path,
                'brief_summary' => $request->brief_summary,
                'outstanding_issues' => $request->outstanding_issues,
                'communication_summary' => $request->communication_summary,
                'final_case_notes' => $request->final_case_notes,
                'digital_signature' => $request->digital_signature
            ]);

            // Update case status to closed
            $case = InsuranceClaim::findOrFail($request->claim_id);
            $oldStatus = $case->status;
            $case->update(['status' => 'closed']);

            // Log the status change
            StatusChange::create([
                'insurance_claim_id' => $request->claim_id,
                'from_status' => $oldStatus,
                'to_status' => 'closed',
                'user_id' => auth()->id()
            ]);

            return response()->json([
                'status' => 'success',
                'message' => 'Case has been closed successfully'
            ]);

        } catch (\Exception $e) {
            \Log::error('Error closing case: ' . $e->getMessage());
            return response()->json([
                'status' => 'error',
                'message' => 'An error occurred while closing the case'
            ], 500);
        }
    }

    public function reopenCase(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'claim_id' => 'required|exists:insurance_claims,id'
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => 'error',
                    'message' => 'Validation failed',
                    'errors' => $validator->errors()
                ], 422);
            }

            // Update case status to open
            $case = InsuranceClaim::findOrFail($request->claim_id);
            $oldStatus = $case->status;
            $case->update([
                'status' => 'open',
                'reopen' => 1
            ]);

            // Log the status change
            StatusChange::create([
                'insurance_claim_id' => $request->claim_id,
                'from_status' => $oldStatus,
                'to_status' => 'open',
                'user_id' => auth()->id()
            ]);

            return response()->json([
                'status' => 'success',
                'message' => 'Case has been reopened successfully'
            ]);

        } catch (\Exception $e) {
            \Log::error('Error reopening case: ' . $e->getMessage());
            return response()->json([
                'status' => 'error',
                'message' => 'An error occurred while reopening the case'
            ], 500);
        }
    }

    public function downloadAttachments($claimId)
    {
        try {
            $claim = InsuranceClaim::findOrFail($claimId);
            $attachments = ClaimAttachment::where('insurance_claim_id', $claimId)->get();

            if ($attachments->isEmpty()) {
                return redirect()->back()->with('error', 'No attachments found for this claim.');
            }

            $zipFileName = 'claim_' . $claimId . '_attachments_' . time() . '.zip';
            $zipPath = storage_path('app/temp/' . $zipFileName);

            // Create temp directory if it doesn't exist
            if (!File::exists(storage_path('app/temp'))) {
                File::makeDirectory(storage_path('app/temp'), 0755, true);
            }

            $zip = new ZipArchive();
            if ($zip->open($zipPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
                foreach ($attachments as $attachment) {
                    $filePath = storage_path('app/public/' . $attachment->file_path);
                    if (File::exists($filePath)) {
                        // Add file to zip with original name
                        $zip->addFile($filePath, $attachment->original_name);
                    }
                }
                $zip->close();

                // Download the zip file
                return response()->download($zipPath)->deleteFileAfterSend(true);
            }

            return redirect()->back()->with('error', 'Could not create zip file.');

        } catch (\Exception $e) {
            \Log::error('Error downloading attachments: ' . $e->getMessage());
            return redirect()->back()->with('error', 'An error occurred while downloading attachments.');
        }
    }

    public function downloadClosureDocuments($claimId)
    {
        try {
            $closure = CaseClosure::where('insurance_claim_id', $claimId)->first();

            if (!$closure || !$closure->supporting_documentation) {
                return redirect()->back()->with('error', 'No closure documents found for this claim.');
            }

            $documents = json_decode($closure->supporting_documentation, true);
            if (!is_array($documents) || empty($documents)) {
                return redirect()->back()->with('error', 'No closure documents found for this claim.');
            }

            $zipFileName = 'claim_' . $claimId . '_closure_docs_' . time() . '.zip';
            $zipPath = storage_path('app/temp/' . $zipFileName);

            // Create temp directory if it doesn't exist
            if (!File::exists(storage_path('app/temp'))) {
                File::makeDirectory(storage_path('app/temp'), 0755, true);
            }

            $zip = new ZipArchive();
            if ($zip->open($zipPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
                foreach ($documents as $doc) {
                    $filePath = storage_path('app/public/' . $doc);
                    if (File::exists($filePath)) {
                        $zip->addFile($filePath, basename($doc));
                    }
                }
                $zip->close();

                return response()->download($zipPath)->deleteFileAfterSend(true);
            }

            return redirect()->back()->with('error', 'Could not create zip file.');

        } catch (\Exception $e) {
            \Log::error('Error downloading closure documents: ' . $e->getMessage());
            return redirect()->back()->with('error', 'An error occurred while downloading closure documents.');
        }
    }

    public function generateCaseDetailsPdf($id)
    {
        try {
            $case = InsuranceClaim::findOrFail($id);
            $additionalInfo = AdditionalInformation::where('insurance_claim_id', $id)->first();
            $caseClosure = CaseClosure::with('user')->where('insurance_claim_id', $id)->first();

            $pdf = PDF::loadView('pdf.case-details', compact('case', 'additionalInfo', 'caseClosure'));
            
            return $pdf->download('case_details_'.$id.'.pdf');

        } catch (\Exception $e) {
            \Log::error('Error generating PDF: ' . $e->getMessage());
            return redirect()->back()->with('error', 'An error occurred while generating the PDF.');
        }
    }

    public function deleteAttachment($id)
{
    $attachment = ClaimAttachment::findOrFail($id);
    Storage::delete($attachment->file_path);
    $attachment->delete();
    
    return redirect()->back()->with('success', 'File deleted successfully');
}
}
